﻿using MediatR;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Hotline.Application.FlowEngine;
using Hotline.DataSharing.Province.Notifications;
using Hotline.Orders;
using Hotline.Share.Dtos.FlowEngine;
using Hotline.Share.Enums.FlowEngine;
using Hotline.Share.Enums.Order;
using XF.Domain.Exceptions;
using XF.Domain.Repository;

namespace Hotline.Application.Handlers.Order
{
    public class DelayProvinceResultNotifyHandler : INotificationHandler<DelayProvinceResultNotify>
    {
        private readonly IRepository<OrderDelay> _orderDelayRepository;
        private readonly IWorkflowApplication _workflowApplication;

        public DelayProvinceResultNotifyHandler(
            IRepository<OrderDelay> orderDelayRepository,
            IWorkflowApplication workflowApplication)
        {
            _orderDelayRepository = orderDelayRepository;
            _workflowApplication = workflowApplication;
        }

        /// <summary>Handles a notification</summary>
        /// <param name="notification">The notification</param>
        /// <param name="cancellationToken">Cancellation token</param>
        public async Task Handle(DelayProvinceResultNotify notification, CancellationToken cancellationToken)
        {
            var dto = notification;
            var orderDelay = await _orderDelayRepository.Queryable()
                .Includes(d => d.Order)
                .Includes(d => d.Workflow)
                .FirstAsync(d => d.Order.ProvinceNo == dto.No && d.DelayState == EDelayState.Examining, cancellationToken);
            if (orderDelay is null)
                throw new UserFriendlyException("无效省工单编号");

            //todo 延期业务

            if (dto.IsPass)
            {
                //todo workflow优化
                var nextDtos =
                    await _workflowApplication.GetNextStepsAsync(orderDelay.WorkflowId, cancellationToken);
                var nextStep = nextDtos.Steps.FirstOrDefault(d => d.StepType == EStepType.End);
                if (nextStep is null)
                    throw new UserFriendlyException("未查询到结束节点");

                var nextDto = new NextWorkflowDto
                {
                    WorkflowId = orderDelay.WorkflowId,
                    //StepExpiredTime = orderDelay.Workflow.ExpiredTime,
                    NextStepCode = nextStep.Key,
                    NextStepName = nextStep.Value,
                    NextMainHandler = nextStep.Key,
                    Opinion = dto.Opinion
                };
                await _workflowApplication.NextAsync(nextDto, cancellationToken);
            }
            else
            {
                var rejectDto = new RejectDto
                {
                    WorkflowId = orderDelay.WorkflowId,
                    Opinion = dto.Opinion,
                };
                await _workflowApplication.RejectAsync(rejectDto, cancellationToken);
            }
        }
    }
}
