﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Hotline.DataSharing.Province.Notifications;
using Hotline.Orders;
using Hotline.Settings;
using MediatR;
using Microsoft.AspNetCore.Http;
using XF.Domain.Repository;

namespace Hotline.Application.Handlers.Order
{
    public class ProvinceOrderSuperviseNotifyHandler : INotificationHandler<ProvinceOrderSuperviseNotify>
    {
        private readonly IOrderRepository _orderRepository;
        private readonly IRepository<OrderSupervise> _orderSuperviseRepository;
        private readonly ISystemOrganizeRepository _organizeRepository;

        public ProvinceOrderSuperviseNotifyHandler(
            IOrderRepository orderRepository,
            IRepository<OrderSupervise> orderSuperviseRepository,
            ISystemOrganizeRepository organizeRepository)
        {
            _orderRepository = orderRepository;
            _orderSuperviseRepository = orderSuperviseRepository;
            _organizeRepository = organizeRepository;
        }

        /// <summary>Handles a notification</summary>
        /// <param name="notification">The notification</param>
        /// <param name="cancellationToken">Cancellation token</param>
        public async Task Handle(ProvinceOrderSuperviseNotify notification, CancellationToken cancellationToken)
        {
            var dto = notification.ProvinceOrderSuperviseDto;
            var order = await _orderRepository.Queryable().Where(x => x.ProvinceNo == dto.CaseSerial).FirstAsync();
            var org = await _organizeRepository.Queryable().Where(x => x.Name == dto.SuperviseRsvDept).FirstAsync();
            var model = new OrderSupervise
            {
                OrderId = order.Id,
                ReplyLimitTime = dto.SuperviseReturnDate,
                OrgId = org.Id,
                OrgName = org.Name,
                LaunchOrgName = dto.SuperviseSendDept,
                State = 0,
                ApplyContent = dto.SuperviseContent,
                CrUser = dto.SupervisePerson,
                SuperviseSerial = dto.SuperviseSerial
            };
            await _orderSuperviseRepository.AddAsync(model, cancellationToken);
        }
    }
}
