﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Hotline.Application.FlowEngine;
using Hotline.DataSharing.Province.Notifications;
using Hotline.Orders;
using Hotline.Share.Dtos.FlowEngine;
using Hotline.Share.Enums.Order;
using MediatR;
using Microsoft.AspNetCore.Http;
using XF.Domain.Exceptions;
using XF.Domain.Repository;

namespace Hotline.Application.Handlers.Order
{
    public class ScreenProvinceResultNotifyHandler : INotificationHandler<ScreenProvinceResultNotify>
    {
        private readonly IRepository<OrderScreen> _orderScreenRepository;
        private readonly IWorkflowApplication _workflowApplication;

        public ScreenProvinceResultNotifyHandler(
            IRepository<OrderScreen> orderScreenRepository,
            IWorkflowApplication workflowApplication)
        {
            _orderScreenRepository = orderScreenRepository;
            _workflowApplication = workflowApplication;
        }

        /// <summary>Handles a notification</summary>
        /// <param name="notification">The notification</param>
        /// <param name="cancellationToken">Cancellation token</param>
        public async Task Handle(ScreenProvinceResultNotify notification, CancellationToken cancellationToken)
        {
            var dto = notification.ProvinceScreenResult;
            var orderScreen = await _orderScreenRepository.Queryable()
                .Includes(d => d.Order)
                .Includes(d => d.Workflow)
                .FirstAsync(d => d.Order.ProvinceNo == dto.CaseSerial && d.Status == EScreenStatus.Approval, cancellationToken);
            if (orderScreen is null)
                throw new UserFriendlyException("无效省工单编号");
            var pass = "1".Equals(dto.AuditResult);
            if (pass)
            {
                //todo workflow优化
                var nextDtos =
                    await _workflowApplication.GetNextStepsAsync(orderScreen.WorkflowId, cancellationToken);
                var nextStep = nextDtos.Steps.FirstOrDefault(d => d.Key == "end");
                if (nextStep is null)
                    throw new UserFriendlyException("未查询到结束节点");

                var nextDto = new NextWorkflowDto
                {
                    WorkflowId = orderScreen.WorkflowId,
                    NextStepCode = nextStep.Key,
                    NextStepName = nextStep.Value,
                    NextMainHandler = nextStep.Key,
                    Opinion = dto.AuditOpinion
                };
                await _workflowApplication.NextAsync(nextDto, cancellationToken);
            }
            else
            {
                var rejectDto = new RejectDto
                {
                    WorkflowId = orderScreen.WorkflowId,
                    Opinion = dto.AuditOpinion,
                };
                await _workflowApplication.RejectAsync(rejectDto, cancellationToken);
            }
        }
    }
}
